"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HandlerService = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const types_1 = require("../../../types");
const utils_1 = require("../../../utils");
const constants_1 = require("../../../constants");
function isArrayData(data) {
    return Array.isArray(data);
}
function isTypeTransform(pipe) {
    return !(typeof pipe === 'function' && /^\s*class\s+/.test(pipe.toString()));
}
function isMethod(handler, methodName) {
    if (typeof methodName !== 'string')
        return false;
    const methode = Reflect.get(handler, methodName);
    return methode && typeof methode === 'function';
}
let HandlerService = class HandlerService {
    constructor() {
        this.mapInjectPipe = new Map();
    }
    async runRpc(data) {
        if (isArrayData(data)) {
            const result = [];
            for (const item of data) {
                const callRpcResult = await this.callRpc(item, item.id);
                result.push(callRpcResult);
            }
            return result;
        }
        else {
            return this.callRpc(data, data.id);
        }
    }
    async callRpc(rpcData, id) {
        try {
            const result = await this.callHandler(rpcData);
            return {
                ...result,
                id,
            };
        }
        catch (e) {
            if (e instanceof utils_1.RpcError) {
                return (0, utils_1.fromRpcErrorToRpcErrorObject)(e, id);
            }
            return (0, utils_1.fromRpcErrorToRpcErrorObject)((0, utils_1.createError)(types_1.ErrorCodeType.ServerError, e.message), id);
        }
    }
    async callHandler(rpcData) {
        const handler = this.mapHandler.get(rpcData.method.spaceName);
        if (!handler) {
            throw (0, utils_1.createError)(types_1.ErrorCodeType.MethodNotFound, `${rpcData.method.spaceName} not found`);
        }
        const methodName = rpcData.method.methodName;
        if (!isMethod(handler, methodName)) {
            throw (0, utils_1.createError)(types_1.ErrorCodeType.MethodNotFound, `${rpcData.method.spaceName}.${rpcData.method.methodName} not found`);
        }
        const params = await this.getParamsForHandler(handler, methodName, rpcData.params);
        const result = await handler[methodName](...params);
        const { jsonrpc } = rpcData;
        return {
            jsonrpc,
            result,
            id: null,
        };
    }
    async getParamsForHandler(nameSpaceInst, methodeName, params) {
        const pipesMetadata = Reflect.getMetadata(constants_1.JsonRpcMetadataKeyParamPipe, nameSpaceInst.constructor, methodeName);
        const pipeObjectByIndex = new Map();
        for (const item in pipesMetadata) {
            const index = item.split(':').at(-1);
            if (!index) {
                continue;
            }
            pipeObjectByIndex.set(parseInt(index, 10), pipesMetadata[item]);
        }
        const paramsType = Reflect.getMetadata('design:paramtypes', nameSpaceInst.constructor.prototype, methodeName);
        const callbackFunc = async (params, index) => {
            let pipe = pipeObjectByIndex.get(index);
            if (pipe) {
                pipe = await this.getPipeByType(pipe);
            }
            let metatype = {
                type: 'custom',
                data: '',
                metatype: undefined,
            };
            if (paramsType && paramsType[index]) {
                metatype = paramsType[index];
            }
            return {
                pipe,
                metatype,
                params,
                index,
            };
        };
        const iterate = this.asyncIterate.createIterator(params, callbackFunc);
        const argAfterParse = [];
        for await (const paramItem of iterate) {
            const { params, pipe, metatype, index } = paramItem;
            if (!pipe) {
                argAfterParse.push(params);
                continue;
            }
            try {
                const transFromParams = await pipe.transform(params, metatype);
                argAfterParse.push(transFromParams);
            }
            catch (e) {
                throw (0, utils_1.createError)(e instanceof common_1.BadRequestException
                    ? types_1.ErrorCodeType.InvalidParams
                    : types_1.ErrorCodeType.InternalError, e.message, `Argument: #${index}`);
            }
        }
        return argAfterParse;
    }
    async getPipeByType(pipe) {
        let targetPipeTransform;
        if (isTypeTransform(pipe)) {
            targetPipeTransform = pipe;
        }
        else {
            const hasInMap = this.mapInjectPipe.get(pipe);
            if (hasInMap) {
                targetPipeTransform = hasInMap;
            }
            else {
                try {
                    return this.moduleRef.get(pipe, {
                        strict: false,
                    });
                }
                catch (e) {
                    targetPipeTransform = await this.moduleRef.create(pipe);
                    this.mapInjectPipe.set(pipe, targetPipeTransform);
                }
            }
        }
        return targetPipeTransform;
    }
};
exports.HandlerService = HandlerService;
tslib_1.__decorate([
    (0, common_1.Inject)(constants_1.MAP_HANDLER),
    tslib_1.__metadata("design:type", Map)
], HandlerService.prototype, "mapHandler", void 0);
tslib_1.__decorate([
    (0, common_1.Inject)(core_1.ModuleRef),
    tslib_1.__metadata("design:type", core_1.ModuleRef)
], HandlerService.prototype, "moduleRef", void 0);
tslib_1.__decorate([
    (0, common_1.Inject)(constants_1.ASYNC_ITERATOR_FACTORY),
    tslib_1.__metadata("design:type", Object)
], HandlerService.prototype, "asyncIterate", void 0);
exports.HandlerService = HandlerService = tslib_1.__decorate([
    (0, common_1.Injectable)()
], HandlerService);
//# sourceMappingURL=handler.service.js.map